@@ -29,6 +29,12 @@ def pickphase(eqs, para):
         return
 
 
+class SACFileNotFoundError(Exception):
+    def __init__(self, matchkey):
+        self.matchkey = matchkey
+    def __str__(self):
+        print('No sac files found with {}'.format(self.matchkey))
+
 def datestr2regex(datestr):
     pattern = datestr.replace('%Y', r'\d{4}')
     pattern = pattern.replace('%m', r'\d{2}')
@@ -76,6 +82,8 @@ def match_eq(eq_lst, pathname, stla, stlo, ref_comp='Z', suffix='SAC', offset=No
              tolerance=210, dateformat='%Y.%j.%H.%M.%S', switchEN=False, reverseE=False, reverseN=False):
     pattern = datestr2regex(dateformat)
     ref_eqs = glob.glob(join(pathname, '*{0}*{1}'.format(ref_comp, suffix)))
+    if len(ref_eqs) == 0:
+        raise SACFileNotFoundError(join(pathname, '*{0}*{1}'.format(ref_comp, suffix)))
     sac_files = []
     for ref_sac in ref_eqs:
         try:
@@ -89,7 +97,7 @@ def match_eq(eq_lst, pathname, stla, stlo, ref_comp='Z', suffix='SAC', offset=No
                 tr = obspy.read(ref_sac)[0]
             except TypeError:
                 continue
-            sac_files.append([datestr, tr.stats.starttime, tr.stats.sac.o])
+            sac_files.append([datestr, tr.stats.starttime-tr.stats.sac.b, tr.stats.sac.o])
         else:
             raise TypeError('offset should be int or float type')
     new_col = ['dis', 'bazi', 'data']
@@ -137,7 +145,6 @@ def CfgParser(cfg_file):
         raise FileNotFoundError('Cannot open configure file %s' % cfg_file)
     pa.datapath = cf.get('path', 'datapath')
     pa.rfpath = cf.get('path', 'rfpath')
-    pa.imagepath = cf.get('path', 'imagepath')
     logpath = cf.get('path', 'catalogpath')
     if logpath != '':
         pa.catalogpath = cf.get('path', 'catalogpath')
@@ -160,6 +167,13 @@ def CfgParser(cfg_file):
                 pa.__dict__[key] = cf.getboolean(sec, 'only_r')
             elif key == 'criterion':
                 pa.criterion = value
+            elif key == 'decon_method':
+                pa.decon_method = value
+            elif key == 'rmsgate':
+                try:
+                    pa.rmsgate = cf.getfloat(sec, 'rmsgate')
+                except:
+                    pa.rmsgate = None
             else:
                 try:
                     pa.__dict__[key] = float(value)
@@ -265,7 +279,11 @@ class RF(object):
         except Exception as e:
             self.logger.RFlog.error('{0}'.format(e))
             raise e
-        self.logger.RFlog.info('{0} earthquakes matched'.format(self.eqs.shape[0]))
+        if self.eqs.shape[0] == 0:
+            self.logger.RFlog.warning('No earthquakes matched, please check configurations.'.format(self.eqs.shape[0]))
+            sys.exit(1)
+        else:
+            self.logger.RFlog.info('{0} earthquakes matched'.format(self.eqs.shape[0]))
         
     '''
     def save(self, path=''):
@@ -314,7 +332,7 @@ class RF(object):
             row['data'].filter(freqmin=freqmin, freqmax=freqmax, order=order)
 
     def cal_phase(self):
-        self.logger.RFlog.info('Calculate arrivals and ray parameters for all data')
+        self.logger.RFlog.info('Calculate {} arrivals and ray parameters for all data'.format(self.para.phase))
         for _, row in self.eqs.iterrows():
             row['data'].get_arrival(self.model, row['evdp'], row['dis'])
             # row['data'].get_raypara(self.model, row['evdp'], row['dis'])
@@ -356,7 +374,7 @@ class RF(object):
     #         sys.exit(1)
 
     def rotate(self, method='NE->RT', search_inc=False):
-        self.logger.RFlog.info('Rotate {0} phase {1}'.format(self.para.phase, method))
+        self.logger.RFlog.info('Rotate {0} phase to {1}'.format(self.para.phase, method))
         drop_idx = []
         for i, row in self.eqs.iterrows():
             try:
@@ -380,6 +398,7 @@ class RF(object):
         self.logger.RFlog.info('{0} events left after SNR calculation'.format(self.eqs.shape[0]))
 
     def trim(self):
+        self.logger.RFlog.info('Trim waveforms from {:.2f} before P to {:.2f} after P'.format(self.para.time_before, self.para.time_after))
         for _, row in self.eqs.iterrows():
             row['data'].trim(self.para.time_before, self.para.time_after, self.para.phase)
     
@@ -387,11 +406,7 @@ class RF(object):
         pickphase(self.eqs, self.para)
         self.logger.RFlog.info('{0} events left after virtual checking'.format(self.eqs.shape[0]))
 
-    def deconv(self, itmax=None, minderr=None):
-        if itmax is None:
-            itmax = self.para.itmax
-        if minderr is None:
-            minderr = self.para.minderr
+    def deconv(self):
         if self.para.phase == 'P':
             shift = self.para.time_before
             time_after = self.para.time_after
@@ -406,10 +421,15 @@ class RF(object):
         for i, row in self.eqs.iterrows():
             count += 1
             try:
-                row['data'].deconvolute(shift, time_after, self.para.gauss, phase=self.para.phase,
-                                    only_r=self.para.only_r, itmax=itmax, minderr=minderr, target_dt=self.para.target_dt)
-                self.logger.RFlog.info('Iterative Decon {0} ({3}/{4}) iterations: {1}; final RMS: {2:.4f}'.format(
-                    row['data'].datestr, row['data'].it, row['data'].rms[-1], count, self.eqs.shape[0]))
+                row['data'].deconvolute(shift, time_after, method=self.para.decon_method, f0=self.para.gauss, phase=self.para.phase,
+                                    only_r=self.para.only_r, itmax=self.para.itmax, minderr=self.para.minderr,
+                                    wlevel=self.para.wlevel, target_dt=self.para.target_dt)
+                if self.para.decon_method == 'iter':
+                    self.logger.RFlog.info('Iterative Decon {0} ({3}/{4}) iterations: {1}; final RMS: {2:.4f}'.format(
+                        row['data'].datestr, row['data'].it, row['data'].rms[-1], count, self.eqs.shape[0]))
+                elif self.para.decon_method == 'water':
+                    self.logger.RFlog.info('Water level Decon {} ({}/{}); RMS: {:.4f}'.format(
+                        row['data'].datestr, count, self.eqs.shape[0], row['data'].rms))
             except Exception as e:
                 self.logger.RFlog.error('{}: {}'.format(row['data'].datestr, e))
                 drop_lst.append(i)
@@ -427,9 +447,12 @@ class RF(object):
             pass
         good_lst = []
 
-        self.logger.RFlog.info('Save RFs with criterion of {}'.format(criterion))
+        if self.para.rmsgate is not None:
+            self.logger.RFlog.info('Save RFs with final RMS less than {:.2f} and criterion of {}'.format(self.para.rmsgate, criterion))
+        else:
+            self.logger.RFlog.info('Save RFs with and criterion of {}'.format(criterion))
         for i, row in self.eqs.iterrows():
-            if row['data'].judge_rf(shift, npts, criterion=criterion):
+            if row['data'].judge_rf(shift, npts, criterion=criterion, rmsgate=self.para.rmsgate):
                 row['data'].saverf(self.para.rfpath, evtstr=row['date'].strftime('%Y.%j.%H.%M.%S'), phase=self.para.phase, shift=shift,
                                    evla=row['evla'], evlo=row['evlo'], evdp=row['evdp'], baz=row['bazi'],
                                    mag=row['mag'], gcarc=row['dis'], gauss=self.para.gauss, only_r=self.para.only_r)
@@ -442,11 +465,12 @@ def prf():
     parser = argparse.ArgumentParser(description="Calculating RFs for single station")
     parser.add_argument('cfg_file', type=str, help='Path to RF configure file')
     parser.add_argument('-l', help="use local catalog. Default is false", dest='islocal', action='store_true')
-    parser.add_argument('-r', help='Reverse components: EN, E or N', dest='comp', default=None, type=str)
-    parser.add_argument('-s', help='Switch East and North components', dest='isswitch', action='store_true')
-    parser.add_argument('-b', help='Correct back-azimuth with minimal '
-                                   'energy of T component. "baz" is specified '
-                                   'as half-searching range. Default value is 90 deg',
+    parser.add_argument('-r', help='Reverse components: EN, E or N', dest='comp',
+                        metavar='N|E|NE', default=None, type=str)
+    parser.add_argument('-s', help='Switch the East and North components', dest='isswitch', action='store_true')
+    parser.add_argument('-b', help='Correct back-azimuth. \nIf "baz" is specified, the corr_baz = raw_baz + baz. \n'
+                                   'If no arguments following -b Back-azimuth will be corrected with minimal '
+                                   'energy of T component. The searching range is raw_baz +/- 90',
                                    dest='baz', nargs='?', const=0, type=float)
     arg = parser.parse_args()
     if arg.comp is not None:
@@ -485,7 +509,13 @@ def prf():
     else:
         pass
     pjt.trim()
-    pjt.rotate()
+    targ_comp = ''.join(sorted(pjt.para.comp.upper()))
+    if targ_comp == 'RTZ':
+        pjt.rotate(method='NE->RT')
+    elif targ_comp == 'LQT':
+        pjt.rotate(method='ZNE->LQT')
+    else:
+        raise ValueError('comp must be in RTZ or LQT.')
     pjt.deconv()
     pjt.saverf()
 
